import opengl;
import openglu;

import SDL;
//import SDL_pad;

import texture;
import log;
import myio;
import mymath;
import vector;

int screenWidth_, screenHeight_;
bool done_;

void initSDL() {
    Log.info("init SDL");

    int ret = SDL_Init(SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE);
    if (ret != 0) Log.errorSDL("cannot init SDL");

    SDL_VideoInfo* info = SDL_GetVideoInfo();

    int bpp = 32;
    char[] bppMsg;

    if (bpp == -1) {
        bpp = info.vfmt.BitsPerPixel;
        Log.info("using the \"best\" bpp: " ~ tostr(bpp));
    }
    else if (bpp == 0) {
        bpp = info.vfmt.BitsPerPixel;
        Log.info("using current bpp");
    }
    else {
        Log.info("using user set bpp: " ~ tostr(bpp));
    }

    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );  
    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );  
//        SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

    screenWidth_ = 640;
    screenHeight_ = 480;
    int screenFull = 0;
    int flags = SDL_OPENGL;
    if (screenFull) {
        flags |= SDL_FULLSCREEN;
    }

    Log.info("set video mode " ~
             "size = " ~ tostr(screenWidth_) ~
             "x" ~ tostr(screenHeight_) ~
             ", bpp = " ~ tostr(bpp) ~
             ", flags = " ~ tostr(flags));
    if (SDL_SetVideoMode(screenWidth_, screenHeight_,
                         bpp, flags) == null)
    {
        Log.errorSDL("cannot create screen");
    }

    info = SDL_GetVideoInfo();
    Log.info("set bpp: " ~ tostr(cast(int)info.vfmt.BitsPerPixel));

//    Pad_Init();
//    Pad_LoadDefault("key.conf");

    if (flags & SDL_FULLSCREEN) {
        SDL_ShowCursor(SDL_DISABLE);
    }

    SDL_WM_SetCaption(`fire`, null);
}

void initGL() {
    glClearColor(0, 0, 0, 0);
    glViewport(0, 0, screenWidth_, screenHeight_);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    float asp = cast(float)screenWidth_ / screenHeight_;
    gluPerspective(45.0, asp, 0.1, 1000);

    glEnable(GL_BLEND);
//    glEnable(GL_LINE_SMOOTH);

    glBlendFunc(GL_SRC_ALPHA, GL_ONE);

    glMatrixMode(GL_MODELVIEW);

    // from PARSEC47
    glDisable(GL_LIGHTING);
    glDisable(GL_CULL_FACE);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_COLOR_MATERIAL);

    Texture.initTextures();

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

    if (glGetError()) {
        Log.error(tostr(gluErrorString(glGetError())));
    }

//    initLetter();
}

Vec3[] pos, spd;

int main() {
    Log.init();

    initSDL();
    initGL();

    pos.length = 10000;
    spd.length = 10000;

    done_ = false;
    int t = 0;

    while (!done_) {
        SDL_Event ev;
        while (SDL_PollEvent(&ev)) {
            if (ev.type == SDL_QUIT) done_ = true;
            if (ev.type == SDL_KEYDOWN) {
                SDLKey k = ev.key.keysym.sym;
                if (k == SDLK_ESCAPE || k == SDLK_q) {
                    done_ = true;
                }
            }
        }

        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
        gluLookAt(320, 240, -500, 320, 240, 0, 0, -1, 0);

        for (int i = 0; i < 10; i++) {
            int mk = t % 10000;
            float l = rnd(100.0);
            float a = rnd(2*PI);
            pos[mk].x = cos(a)*l+320;
            pos[mk].y = sin(a)*l+240;
            pos[mk].z = -400;
            spd[mk].x = rnd(1.0)-0.5;
            spd[mk].y = rnd(1.0)-0.5;
            spd[mk].z = -rnd(5.0)+1.0;
            t++;
        }

        int max = (t > 10000) ? 10000 : t;
        for (int i = 0; i < max; i++) {
            pos[i] += spd[i];
        }

        glEnable(GL_TEXTURE_2D);
        GLfloat[] texcoord = Texture.prepare(TextureID.TEX_PARTICLE);
        for (int i = 0; i < max; i++) {
            glPushMatrix();
            glTranslatef(pos[i].x, pos[i].y, pos[i].z);

            glColor4f(1, 0.2, 0.2, 0.2);

            float tx = 16;
            float ty = 16;
            glBegin(GL_QUADS);
            glTexCoord3f(texcoord[0], texcoord[1], 0);
            glVertex3f(-tx, -ty, 0);
            glTexCoord3f(texcoord[0], texcoord[3], 0);
            glVertex3f(-tx, ty, 0);
            glTexCoord3f(texcoord[2], texcoord[3], 0);
            glVertex3f(tx, ty, 0);
            glTexCoord3f(texcoord[2], texcoord[1], 0);
            glVertex3f(tx, -ty, 0);
            glEnd();

            glPopMatrix();
        }

        SDL_GL_SwapBuffers();

        t++;

        SDL_Delay(10);
    }

    return 0;
}

