private import SDL;
private import opengl;
private import std.c.stdio;
private import std.c.stdlib;
private import std.string;

private import log;

// rough texture manager
enum TextureID {
    TEX_PARTICLE,
/+ /*TEX_GL, TEX_GL_EFF,*/ TEX_BARRIER, TEX_SWORD,
    TEX_PSYNO,
+/
    TEXTURE_NUM
};

class Texture {
    private uint id_;
    private float texcoord_[4];

    /* Quick utility function for texture creation */
    int power_of_two(int input) {
        int value = 1;

        while ( value < input ) {
            value <<= 1;
        }
        return value;
    }

    this(char[] filename) {
        Log.info(filename ~ ": loading texture");

        SDL_Surface* surf = SDL_LoadBMP(filename);
        SDL_SetColorKey(surf, SDL_SRCCOLORKEY, 0);
        this(surf);
        SDL_FreeSurface(surf);
    }

    this(SDL_Surface *surface) {

        /* Use the surface width and height expanded to powers of 2 */
        int w = power_of_two(surface.w);
        int h = power_of_two(surface.h);
        texcoord_[0] = 0.0f;            /* Min X */
        texcoord_[1] = 0.0f;            /* Min Y */
        texcoord_[2] = cast(GLfloat)surface.w / w;    /* Max X */
        texcoord_[3] = cast(GLfloat)surface.h / h;    /* Max Y */

        // from NeHe19
        Uint8* rowhi, rowlo;
        Uint8* tmpbuf;
        Uint8 tmpch;
        SDL_Surface *image;
        int i, j;

        image = surface;
        if ( image == null ) {
            fprintf(stderr, "Unable to load texture: %s\n", SDL_GetError());
            throw new Error("error");
        }

        /* GL surfaces are upsidedown and RGB, not BGR :-) */
        tmpbuf = cast(Uint8 *)malloc(image.pitch);
        if ( tmpbuf == null ) {
            fprintf(stderr, "Out of memory\n");
            throw new Error("error");
        }
        rowhi = cast(Uint8 *)image.pixels;
        rowlo = rowhi + (image.h * image.pitch) - image.pitch;
        for ( i=0; i<image.h/2; ++i ) {
            for ( j=0; j<image.w; ++j ) {
                tmpch = rowhi[j*3];
                rowhi[j*3] = rowhi[j*3+2];
                rowhi[j*3+2] = tmpch;
                tmpch = rowlo[j*3];
                rowlo[j*3] = rowlo[j*3+2];
                rowlo[j*3+2] = tmpch;
            }
            memcpy(tmpbuf, rowhi, image.pitch);
            memcpy(rowhi, rowlo, image.pitch);
            memcpy(rowlo, tmpbuf, image.pitch);
            rowhi += image.pitch;
            rowlo -= image.pitch;
        }
        free(tmpbuf);

        GLuint texture;

        /* Create an OpenGL texture for the image */
        glGenTextures(1, &texture);
        glBindTexture(GL_TEXTURE_2D, texture);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexImage2D(GL_TEXTURE_2D,
                     0,
                     3,
                     image.w, image.h,
                     0,
                     GL_RGB,
                     GL_UNSIGNED_BYTE,
                     image.pixels);

        id_ = texture;
    }

    ~this() {
        deleteTexture();
    }

    void bind() {
        glBindTexture(GL_TEXTURE_2D, id_);
    }

    GLfloat[] texcoord() {
        return texcoord_;
    }

    void deleteTexture() {
        glDeleteTextures(1, &id_);
    }

    static Texture textures[TextureID.TEXTURE_NUM];
    static char[][TextureID.TEXTURE_NUM] textureNames = [
        `particle.bmp`,
//        `barrier.bmp`,
//        `sword.bmp`,
//        `psyno.bmp`,
        ];

    static void initTextures() {
        foreach (int i, char[] n; textureNames) {
            textures[i] = new Texture(n);
        }
    }

    static GLfloat[] prepare(TextureID id) {
        glEnable(GL_TEXTURE_2D);
        Texture tex = textures[id];
        tex.bind();
        return tex.texcoord;
    }

}

